/*******************************************************************************
***  This file is part of Pecunia.                                           ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020, 2021, 2023, 2024, 2025            ***
***  John Schneiderman <Licencing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Lesser General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Lesser General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Lesser General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#ifndef PECUNIA_ALGORITHM_HPP_
#define PECUNIA_ALGORITHM_HPP_

#include <functional>

#include "Money.hpp"


namespace pecunia
{

/**
 * @brief Sorts the container using a less-than comparison operator. The sort is not stable.
 *
 * @tparam ContainerType The type of container holding the monetary values to sort. The container
 * must have random access iterator support. The contained type must support the operators
 * less-than, greater-than, and have a swap function.
 *
 * @param container The container holding the monetary values to sort.
 * @param swaperator A function that will swap the contained values using no monetary conversion in
 * the process.
 */
template<typename ContainerType>
void sort(
	ContainerType& container,
	const std::function<
		void(typename ContainerType::value_type&, typename ContainerType::value_type&)
	>& swaperator = pecunia::swap
);

}

#include "Algorithm.tpp"

#endif
